'''
=============================
Linkage accuracy: Kelp_70_3_138_combined_linkages
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	44	1	43	0	43(100.0)
class	44	1	41	0	41(95.35)
order	44	1	35	0	35(81.4)
family	44	1	34	0	34(79.07)
genus	44	1	23	5	28(65.12)
=============================
linked MAGs: 18




Linkage accuracy: manual_linkages
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	55	1	54	0	54(100.0)
class	55	1	52	0	52(96.3)
order	55	1	46	0	46(85.19)
family	55	1	44	0	44(81.48)
genus	55	1	30	6	36(66.67)
=============================
linked MAGs: 23


Linkage accuracy: mock_final_op
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	60	1	59	0	59(100.0)
class	60	1	57	0	57(96.61)
order	60	1	52	0	52(88.14)
family	60	1	50	0	50(84.75)
genus	60	1	36	6	42(71.19)
=============================
linked MAGs: 26


=============================
Linkage accuracy: mock_final_op
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	60	1	59	0	59(100.0)
class	60	1	57	0	57(96.61)
order	60	1	52	0	52(88.14)
family	60	1	50	0	50(84.75)
genus	60	1	36	6	42(71.19)
=============================
linked MAGs: 26

10_5
=============================
Linkage accuracy: mock_final_op
Rank	Links	NA	Correct	Unknown	Total(Accuracy)
phylum	63	1	62	0	62(100.0)
class	63	1	60	0	60(96.77)
order	63	1	54	0	54(87.1)
family	63	1	52	0	52(83.87)
genus	63	1	39	5	44(70.97)
=============================
linked MAGs: 26

5_3
=============================
Linkage accuracy: mock_final_op
Rank	All	NA	Correct	Unknown	Total(Accuracy)
phylum	79	3	76	0	76(100.0)
class	79	3	74	0	74(97.37)
order	79	3	67	0	67(88.16)
family	79	3	65	0	65(85.53)
genus	79	3	48	9	57(75.0)
=============================
linked MAGs: 28

3_2
=============================
Linkage accuracy: mock_final_op
Rank	All	NA	Correct	Unknown	Total(Accuracy)
phylum	83	3	80	0	80(100.0)
class	83	3	78	0	78(97.5)
order	83	3	71	0	71(88.75)
family	83	3	69	0	69(86.25)
genus	83	3	52	9	61(76.25)
=============================
linked MAGs: 28

3_1
=============================
Linkage accuracy: mock_final_op
Rank	All	NA	Correct	Unknown	Total(Accuracy)
phylum	102	3	99	0	99(100.0)
class	102	3	97	0	97(97.98)
order	102	3	90	0	90(90.91)
family	102	3	88	0	88(88.89)
genus	102	3	61	17	78(78.79)
=============================
linked MAGs: 32

2_1
=============================
Linkage accuracy: mock_final_op
Rank	All	NA	Correct	Unknown	Total(Accuracy)
phylum	105	3	102	0	102(100.0)
class	105	3	99	0	99(97.06)
order	105	3	92	0	92(90.2)
family	105	3	90	0	90(88.24)
genus	105	3	62	18	80(78.43)
=============================
linked MAGs: 35


MarkerGene,GenomicSeq,Number
MarkerGene__Kelp_SILVA138_id99_subsample_10_136,GenomicSeq__Refined_57,84
MarkerGene__Kelp_SILVA138_id99_subsample_75_668,GenomicSeq__Refined_26,51
MarkerGene__Kelp_SILVA138_id99_subsample_25_214,GenomicSeq__Refined_37,43
MarkerGene__Kelp_SILVA138_id99_subsample_75_314,GenomicSeq__Refined_37,42
MarkerGene__Kelp_SILVA138_id99_subsample_100_376,GenomicSeq__Refined_37,41
MarkerGene__Kelp_SILVA138_id99_subsample_100_373,GenomicSeq__Refined_37,40
MarkerGene__Kelp_SILVA138_id99_subsample_50_289,GenomicSeq__Refined_37,40
MarkerGene__Kelp_SILVA138_id99_subsample_10_229,GenomicSeq__Refined_26,34
MarkerGene__Kelp_SILVA138_id99_subsample_25_253,GenomicSeq__Refined_23,31
MarkerGene__Kelp_SILVA138_id99_subsample_25_5,GenomicSeq__Refined_34,31
MarkerGene__Kelp_SILVA138_id99_subsample_100_742,GenomicSeq__Refined_23,27
MarkerGene__Kelp_SILVA138_id99_subsample_25_216,GenomicSeq__Refined_37,25
MarkerGene__Kelp_SILVA138_id99_subsample_100_500,GenomicSeq__Refined_23,24
MarkerGene__Kelp_SILVA138_id99_subsample_75_312,GenomicSeq__Refined_37,23
MarkerGene__Kelp_SILVA138_id99_subsample_25_189,GenomicSeq__Refined_36,21
MarkerGene__Kelp_SILVA138_id99_subsample_5_84,GenomicSeq__Refined_37,20
MarkerGene__Kelp_SILVA138_id99_subsample_50_256,GenomicSeq__Refined_27,20
MarkerGene__Kelp_SILVA138_id99_subsample_75_442,GenomicSeq__Refined_47,19
MarkerGene__Kelp_SILVA138_id99_subsample_10_124,GenomicSeq__Refined_27,18
MarkerGene__Kelp_SILVA138_id99_subsample_100_789,GenomicSeq__Refined_26,17
MarkerGene__Kelp_SILVA138_id99_subsample_75_573,GenomicSeq__Refined_42,16
MarkerGene__Kelp_SILVA138_id99_subsample_50_390,GenomicSeq__Refined_47,15
MarkerGene__Kelp_SILVA138_id99_subsample_50_553,GenomicSeq__Refined_26,15
MarkerGene__Kelp_SILVA138_id99_subsample_75_306,GenomicSeq__Refined_20,14
MarkerGene__Kelp_SILVA138_id99_subsample_50_287,GenomicSeq__Refined_37,14
MarkerGene__Kelp_SILVA138_id99_subsample_10_138,GenomicSeq__Refined_37,13
MarkerGene__Kelp_SILVA138_id99_subsample_25_188,GenomicSeq__Refined_19,13
MarkerGene__Kelp_SILVA138_id99_subsample_100_336,GenomicSeq__Refined_36,11
MarkerGene__Kelp_SILVA138_id99_subsample_50_239,GenomicSeq__Refined_37,11
MarkerGene__Kelp_SILVA138_id99_subsample_100_365,GenomicSeq__Refined_20,11
MarkerGene__Kelp_SILVA138_id99_subsample_25_212,GenomicSeq__Refined_20,11
MarkerGene__Kelp_SILVA138_id99_subsample_100_662,GenomicSeq__Refined_3,8
MarkerGene__Kelp_SILVA138_id99_subsample_100_644,GenomicSeq__Refined_3,8
MarkerGene__Kelp_SILVA138_id99_subsample_75_538,GenomicSeq__Refined_3,7
MarkerGene__Kelp_SILVA138_id99_subsample_100_602,GenomicSeq__Refined_62,7
MarkerGene__Kelp_SILVA138_id99_subsample_75_502,GenomicSeq__Refined_62,7
MarkerGene__Kelp_SILVA138_id99_subsample_25_393,GenomicSeq__Refined_26,7
MarkerGene__Kelp_SILVA138_id99_subsample_100_785,GenomicSeq__Refined_17,6
MarkerGene__Kelp_SILVA138_id99_subsample_50_568,GenomicSeq__Refined_50,6
MarkerGene__Kelp_SILVA138_id99_subsample_75_409,GenomicSeq__Refined_9,6
MarkerGene__Kelp_SILVA138_id99_subsample_25_300,GenomicSeq__Refined_56,5
MarkerGene__Kelp_SILVA138_id99_subsample_100_538,GenomicSeq__Refined_47,5
MarkerGene__Kelp_SILVA138_id99_subsample_75_498,GenomicSeq__Refined_56,5

'''

'''
MarkerGene,GenomicSeq,Number
MarkerGene__Kelp_SILVA138_id99_subsample_10_136,GenomicSeq__Refined_57,165
MarkerGene__Kelp_SILVA138_id99_subsample_25_189,GenomicSeq__Refined_36,117
MarkerGene__Kelp_SILVA138_id99_subsample_50_256,GenomicSeq__Refined_27,104
MarkerGene__Kelp_SILVA138_id99_subsample_75_442,GenomicSeq__Refined_47,98
MarkerGene__Kelp_SILVA138_id99_subsample_75_668,GenomicSeq__Refined_26,85
MarkerGene__Kelp_SILVA138_id99_subsample_75_573,GenomicSeq__Refined_42,73
MarkerGene__Kelp_SILVA138_id99_subsample_10_124,GenomicSeq__Refined_27,69
MarkerGene__Kelp_SILVA138_id99_subsample_50_553,GenomicSeq__Refined_26,62
MarkerGene__Kelp_SILVA138_id99_subsample_75_306,GenomicSeq__Refined_20,50
MarkerGene__Kelp_SILVA138_id99_subsample_75_277,GenomicSeq__Refined_27,50
MarkerGene__Kelp_SILVA138_id99_subsample_25_214,GenomicSeq__Refined_37,50
MarkerGene__Kelp_SILVA138_id99_subsample_75_314,GenomicSeq__Refined_37,49
MarkerGene__Kelp_SILVA138_id99_subsample_100_531,GenomicSeq__Refined_47,48
MarkerGene__Kelp_SILVA138_id99_subsample_100_376,GenomicSeq__Refined_37,47
MarkerGene__Kelp_SILVA138_id99_subsample_100_373,GenomicSeq__Refined_37,47
MarkerGene__Kelp_SILVA138_id99_subsample_50_289,GenomicSeq__Refined_37,47
MarkerGene__Kelp_SILVA138_id99_subsample_25_5,GenomicSeq__Refined_34,47
MarkerGene__Kelp_SILVA138_id99_subsample_50_115,GenomicSeq__Refined_19,46
MarkerGene__Kelp_SILVA138_id99_subsample_10_229,GenomicSeq__Refined_26,45
MarkerGene__Kelp_SILVA138_id99_subsample_100_336,GenomicSeq__Refined_36,44
MarkerGene__Kelp_SILVA138_id99_subsample_50_239,GenomicSeq__Refined_37,36
MarkerGene__Kelp_SILVA138_id99_subsample_25_253,GenomicSeq__Refined_23,31
MarkerGene__Kelp_SILVA138_id99_subsample_75_312,GenomicSeq__Refined_37,30
MarkerGene__Kelp_SILVA138_id99_subsample_50_390,GenomicSeq__Refined_47,30
MarkerGene__Kelp_SILVA138_id99_subsample_100_742,GenomicSeq__Refined_23,29
MarkerGene__Kelp_SILVA138_id99_subsample_100_365,GenomicSeq__Refined_20,27
MarkerGene__Kelp_SILVA138_id99_subsample_100_789,GenomicSeq__Refined_26,26
MarkerGene__Kelp_SILVA138_id99_subsample_25_216,GenomicSeq__Refined_37,25
MarkerGene__Kelp_SILVA138_id99_subsample_100_500,GenomicSeq__Refined_23,24
MarkerGene__Kelp_SILVA138_id99_subsample_5_84,GenomicSeq__Refined_37,23
MarkerGene__Kelp_SILVA138_id99_subsample_25_378,GenomicSeq__Refined_4,23
MarkerGene__Kelp_SILVA138_id99_subsample_50_387,GenomicSeq__Refined_47,22
MarkerGene__Kelp_SILVA138_id99_subsample_5_74,GenomicSeq__Refined_27,22
MarkerGene__Kelp_SILVA138_id99_subsample_50_287,GenomicSeq__Refined_37,19
MarkerGene__Kelp_SILVA138_id99_subsample_100_586,GenomicSeq__Refined_36,17
MarkerGene__Kelp_SILVA138_id99_subsample_75_236,GenomicSeq__Refined_5,16
MarkerGene__Kelp_SILVA138_id99_subsample_100_538,GenomicSeq__Refined_47,14
MarkerGene__Kelp_SILVA138_id99_subsample_25_393,GenomicSeq__Refined_26,13
MarkerGene__Kelp_SILVA138_id99_subsample_75_502,GenomicSeq__Refined_62,13
MarkerGene__Kelp_SILVA138_id99_subsample_10_138,GenomicSeq__Refined_37,13
MarkerGene__Kelp_SILVA138_id99_subsample_50_409,GenomicSeq__Refined_36,11
MarkerGene__Kelp_SILVA138_id99_subsample_25_212,GenomicSeq__Refined_20,11
MarkerGene__Kelp_SILVA138_id99_subsample_5_75,GenomicSeq__Refined_28,11
MarkerGene__Kelp_SILVA138_id99_subsample_100_602,GenomicSeq__Refined_62,10
MarkerGene__Kelp_SILVA138_id99_subsample_50_328,GenomicSeq__Refined_61,10
MarkerGene__Kelp_SILVA138_id99_subsample_75_664,GenomicSeq__Refined_17,10
MarkerGene__Kelp_SILVA138_id99_subsample_100_644,GenomicSeq__Refined_3,9
MarkerGene__Kelp_SILVA138_id99_subsample_75_498,GenomicSeq__Refined_56,9
MarkerGene__Kelp_SILVA138_id99_subsample_25_300,GenomicSeq__Refined_56,9
MarkerGene__Kelp_SILVA138_id99_subsample_50_422,GenomicSeq__Refined_56,8
MarkerGene__Kelp_SILVA138_id99_subsample_10_175,GenomicSeq__Refined_56,8
MarkerGene__Kelp_SILVA138_id99_subsample_50_568,GenomicSeq__Refined_50,8
MarkerGene__Kelp_SILVA138_id99_subsample_100_662,GenomicSeq__Refined_3,8
MarkerGene__Kelp_SILVA138_id99_subsample_75_409,GenomicSeq__Refined_9,8
MarkerGene__Kelp_SILVA138_id99_subsample_5_110,GenomicSeq__Refined_56,7
MarkerGene__Kelp_SILVA138_id99_subsample_100_785,GenomicSeq__Refined_17,7
MarkerGene__Kelp_SILVA138_id99_subsample_100_385,GenomicSeq__Refined_24,6
MarkerGene__Kelp_SILVA138_id99_subsample_100_371,GenomicSeq__Refined_22,6
MarkerGene__Kelp_SILVA138_id99_subsample_100_441,GenomicSeq__Refined_61,6
MarkerGene__Kelp_SILVA138_id99_subsample_50_540,GenomicSeq__Refined_11,5
MarkerGene__Kelp_SILVA138_id99_subsample_75_315,GenomicSeq__Refined_6,5
MarkerGene__Kelp_SILVA138_id99_subsample_75_625,GenomicSeq__Refined_32,5
MarkerGene__Kelp_SILVA138_id99_subsample_50_518,GenomicSeq__Refined_32,5
'''

'''
Kelp_SILVA138_id99_subsample_5_147	Refined_54	42	S2	1	1	1	1	1	d__Bacteria;p__Planctomycetota;c__Planctomycetes;o__Pirellulales;f__Pirellulaceae;g__Mariniblastus;s__	d__Bacteria;p__Planctomycetota;c__Planctomycetes;o__Pirellulales;f__Pirellulaceae;g__Mariniblastus;s__Mariniblastus_fucicola	Bacteria(100.0);Planctomycetota(100.0);Planctomycetes(100.0);Pirellulales(100.0);Pirellulaceae(100.0);Mariniblastus(100.0);Mariniblastus fucicola(100.0)
'''

'''
500
Kelp_SILVA138_id99_subsample_25_303	Refined_48	518	S2
Kelp_SILVA138_id99_subsample_50_418	Refined_31	27	S2
Kelp_SILVA138_id99_subsample_25_389	Refined_42	24	S2
Kelp_SILVA138_id99_subsample_100_694	Refined_31	24	S2
Kelp_SILVA138_id99_subsample_75_575	Refined_31	24	S2
Kelp_SILVA138_id99_subsample_100_575	Refined_14	15	S2
Kelp_SILVA138_id99_subsample_25_264	Refined_49	6	S2
Kelp_SILVA138_id99_subsample_100_515	Refined_49	6	S2
Kelp_SILVA138_id99_subsample_10_228	Refined_54	6	S2
Kelp_SILVA138_id99_subsample_100_487	Refined_31	4	S2
Kelp_SILVA138_id99_subsample_100_784	Refined_42	3	S2
Kelp_SILVA138_id99_subsample_50_487	Refined_31	3	S2
Kelp_SILVA138_id99_subsample_75_388	Refined_63	3	S2
Kelp_SILVA138_id99_subsample_100_459	Refined_63	3	S2

750
Kelp_SILVA138_id99_subsample_25_303	Refined_48	518	S2
Kelp_SILVA138_id99_subsample_50_418	Refined_31	27	S2
Kelp_SILVA138_id99_subsample_25_389	Refined_42	25	S2
Kelp_SILVA138_id99_subsample_100_694	Refined_31	24	S2
Kelp_SILVA138_id99_subsample_75_575	Refined_31	24	S2
Kelp_SILVA138_id99_subsample_100_575	Refined_14	15	S2
Kelp_SILVA138_id99_subsample_100_515	Refined_49	6	S2
Kelp_SILVA138_id99_subsample_25_264	Refined_49	6	S2
Kelp_SILVA138_id99_subsample_10_228	Refined_54	5	S2
Kelp_SILVA138_id99_subsample_100_487	Refined_31	4	S2
Kelp_SILVA138_id99_subsample_50_487	Refined_31	3	S2
Kelp_SILVA138_id99_subsample_100_784	Refined_42	3	S2
Kelp_SILVA138_id99_subsample_75_388	Refined_63	3	S2
Kelp_SILVA138_id99_subsample_100_459	Refined_63	3	S2


1000




'''