

demo_dict = {'a':'1', 'b':2, 'c':3}

demo_set = {'a', 'c'}

set_a = {'1','2',3}
set_b = {3,'5','6'}

set_c = set.union(set_a, set_b)


print(set_a)
print(set_b)
print(set_c)
print(set_a)
print(set_b)