'''MarkerGene	GenomicSeq	Linkage	Step
3_GI_subsample_50_875	Refined_7	2603	S1
3_GI_subsample_100_1832	Refined_7	2508	S1
3_GI_subsample_50_881	Refined_7	2062	S1
3_GI_subsample_100_2289	Refined_7	1791	S1
3_GI_subsample_75_69	Refined_4	1783	S1
3_GI_subsample_100_1336	Refined_4	1752	S1
3_GI_subsample_50_650	Refined_4	1310	S1
3_GI_subsample_75_1732	Refined_7	1303	S1
3_GI_subsample_100_3665	Refined_7	1187	S1
3_GI_subsample_50_882	Refined_7	1173	S1
3_GI_subsample_25_913	Refined_4	924	S1
3_GI_subsample_50_1564	Refined_4	695	S1
3_GI_subsample_75_964	Refined_52	477	S1
3_GI_subsample_50_583	Refined_52	465	S1
3_GI_subsample_75_1207	Refined_43	424	S1
3_GI_subsample_75_304	Refined_29	417	S1
3_GI_subsample_50_580	Refined_52	399	S1
3_GI_subsample_100_3361	Refined_4	387	S1
3_GI_subsample_75_1085	Refined_52	378	S1
3_GI_subsample_50_213	Refined_29	366	S1
3_GI_subsample_100_1366	Refined_42	357	S1
3_GI_subsample_100_1167	Refined_42	340	S1
3_GI_subsample_50_1690	Refined_33	338	S1
3_GI_subsample_75_383	Refined_33	335	S1
3_GI_subsample_75_382	Refined_33	333	S1
3_GI_subsample_75_1091	Refined_43	332	S1
3_GI_subsample_100_1372	Refined_42	331	S1
3_GI_subsample_75_960	Refined_33	297	S1
3_GI_subsample_100_1271	Refined_42	287	S1
3_GI_subsample_75_963	Refined_52	286	S1
3_GI_subsample_75_1206	Refined_43	251	S1
3_GI_subsample_75_1105	Refined_43	249	S1
3_GI_subsample_100_1367	Refined_42	241	S1
3_GI_subsample_25_345	Refined_4	195	S1
3_GI_subsample_100_1437	Refined_31	188	S1
3_GI_subsample_100_1337	Refined_42	183	S1
3_GI_subsample_50_595	Refined_4	173	S1
3_GI_subsample_100_2493	Refined_33	171	S1
3_GI_subsample_75_1086	Refined_52	170	S1
3_GI_subsample_100_2322	Refined_4	168	S1
3_GI_subsample_75_1227	Refined_4	155	S1
3_GI_subsample_75_1075	Refined_31	155	S1
3_GI_subsample_100_1076	Refined_30	147	S1
3_GI_subsample_100_1848	Refined_30	143	S1
3_GI_subsample_100_1847	Refined_30	142	S1
3_GI_subsample_75_1074	Refined_31	129	S1
3_GI_subsample_10_76	Refined_33	127	S1
3_GI_subsample_100_121	Refined_62	121	S1
3_GI_subsample_100_1365	Refined_42	116	S1
3_GI_subsample_50_81	Refined_18	116	S1
3_GI_subsample_75_1078	Refined_31	113	S1
3_GI_subsample_75_1079	Refined_31	107	S1
3_GI_subsample_100_3378	Refined_29	103	S1
3_GI_subsample_25_275	Refined_70	103	S1
3_GI_subsample_100_1192	Refined_11	98	S1
3_GI_subsample_100_1193	Refined_11	98	S1
3_GI_subsample_75_1076	Refined_31	96	S1
3_GI_subsample_75_1195	Refined_52	92	S1
3_GI_subsample_75_961	Refined_52	91	S1
3_GI_subsample_100_317	Refined_2	91	S1
3_GI_subsample_75_188	Refined_2	82	S1
3_GI_subsample_50_424	Refined_42	80	S1
3_GI_subsample_25_484	Refined_95	78	S1
3_GI_subsample_50_663	Refined_31	78	S1
3_GI_subsample_50_113	Refined_21	68	S1
3_GI_subsample_100_1449	Refined_52	67	S1
3_GI_subsample_25_329	Refined_11	67	S1
3_GI_subsample_75_1381	Refined_95	67	S1
3_GI_subsample_25_402	Refined_31	66	S1
3_GI_subsample_50_1037	Refined_52	64	S1
3_GI_subsample_100_1191	Refined_11	63	S1
3_GI_subsample_100_4224	Refined_4	63	S1
3_GI_subsample_25_72	Refined_21	62	S1
3_GI_subsample_100_1338	Refined_42	58	S1
3_GI_subsample_25_582	Refined_52	57	S1
3_GI_subsample_100_313	Refined_2	54	S1
3_GI_subsample_75_226	Refined_2	53	S1
3_GI_subsample_100_279	Refined_2	49	S1
3_GI_subsample_100_3525	Refined_42	46	S1
3_GI_subsample_100_3824	Refined_4	46	S1
3_GI_subsample_100_1395	Refined_36	46	S1
3_GI_subsample_50_285	Refined_25	44	S1
3_GI_subsample_100_1273	Refined_33	43	S1
3_GI_subsample_75_34	Refined_12	43	S1
3_GI_subsample_50_346	Refined_12	43	S1
3_GI_subsample_75_139	Refined_13	40	S1
3_GI_subsample_100_690	Refined_58	40	S1
3_GI_subsample_100_1610	Refined_33	39	S1
3_GI_subsample_75_223	Refined_2	38	S1
3_GI_subsample_100_3152	Refined_42	37	S1
3_GI_subsample_50_811	Refined_95	37	S1
3_GI_subsample_75_1008	Refined_4	36	S1
3_GI_subsample_75_1025	Refined_42	36	S1
3_GI_subsample_100_1393	Refined_36	35	S1
3_GI_subsample_100_187	Refined_2	34	S1
3_GI_subsample_50_869	Refined_95	34	S1
3_GI_subsample_75_1621	Refined_52	33	S1
3_GI_subsample_75_1471	Refined_42	32	S1
3_GI_subsample_75_106	Refined_44	30	S1
3_GI_subsample_75_363	Refined_80	29	S1
3_GI_subsample_75_241	Refined_74	28	S1
3_GI_subsample_100_589	Refined_3	28	S1
3_GI_subsample_25_431	Refined_31	27	S1
3_GI_subsample_100_2066	Refined_4	27	S1
3_GI_subsample_75_2805	Refined_4	26	S1
3_GI_subsample_50_630	Refined_42	25	S1
3_GI_subsample_50_629	Refined_42	25	S1
3_GI_subsample_25_598	Refined_31	25	S1
3_GI_subsample_75_956	Refined_42	24	S1
3_GI_subsample_100_235	Refined_35	23	S1
3_GI_subsample_100_3003	Refined_78	23	S1
3_GI_subsample_10_175	Refined_42	23	S1
3_GI_subsample_75_1701	Refined_76	23	S1
3_GI_subsample_75_1572	Refined_4	22	S1
3_GI_subsample_100_2329	Refined_31	22	S1
3_GI_subsample_50_712	Refined_31	22	S1
3_GI_subsample_100_3364	Refined_4	22	S1
3_GI_subsample_25_863	Refined_4	21	S1
3_GI_subsample_25_763	Refined_4	21	S1
3_GI_subsample_5_85	Refined_59	20	S1
3_GI_subsample_25_565	Refined_4	19	S1
3_GI_subsample_75_1057	Refined_31	18	S1
3_GI_subsample_5_98	Refined_42	18	S1
3_GI_subsample_75_3163	Refined_31	18	S1
3_GI_subsample_100_3344	Refined_42	16	S1
3_GI_subsample_100_2240	Refined_4	16	S1
3_GI_subsample_75_3032	Refined_31	16	S1
3_GI_subsample_50_581	Refined_52	16	S1
3_GI_subsample_75_2486	Refined_42	16	S1
3_GI_subsample_75_2235	Refined_80	15	S1
3_GI_subsample_100_3979	Refined_42	15	S1
3_GI_subsample_50_1945	Refined_4	14	S1
3_GI_subsample_75_2420	Refined_42	14	S1
3_GI_subsample_50_969	Refined_42	14	S1
3_GI_subsample_75_1162	Refined_31	14	S1
3_GI_subsample_75_1055	Refined_31	14	S1
3_GI_subsample_100_4404	Refined_31	13	S1
3_GI_subsample_100_314	Refined_2	13	S1
3_GI_subsample_100_3762	Refined_42	11	S1
3_GI_subsample_100_1548	Refined_72	11	S1
3_GI_subsample_100_2097	Refined_42	11	S1
3_GI_subsample_100_3426	Refined_31	11	S1
3_GI_subsample_100_3714	Refined_31	11	S1
3_GI_subsample_100_1782	Refined_7	11	S1
3_GI_subsample_75_2502	Refined_4	11	S1
3_GI_subsample_50_1728	Refined_31	10	S1
3_GI_subsample_50_370	Refined_69	10	S1
3_GI_subsample_75_1549	Refined_26	10	S1
3_GI_subsample_75_2522	Refined_4	10	S1
3_GI_subsample_100_2560	Refined_2	10	S1
3_GI_subsample_75_2408	Refined_4	9	S1
3_GI_subsample_100_3244	Refined_4	9	S1
3_GI_subsample_100_2419	Refined_2	9	S1
3_GI_subsample_75_1583	Refined_42	9	S1
3_GI_subsample_50_609	Refined_42	8	S1
3_GI_subsample_100_1874	Refined_78	8	S1
3_GI_subsample_100_2390	Refined_42	8	S1
3_GI_subsample_25_378	Refined_42	8	S1
3_GI_subsample_100_4325	Refined_4	8	S1
3_GI_subsample_50_1551	Refined_42	7	S1
3_GI_subsample_100_1396	Refined_36	7	S1
3_GI_subsample_100_1697	Refined_2	7	S1
3_GI_subsample_75_1026	Refined_42	7	S1
3_GI_subsample_50_1500	Refined_4	6	S1
3_GI_subsample_100_2291	Refined_45	6	S1
3_GI_subsample_100_2658	Refined_26	6	S1
3_GI_subsample_50_1600	Refined_31	6	S1
3_GI_subsample_100_4092	Refined_4	6	S1
3_GI_subsample_75_2542	Refined_31	6	S1
3_GI_subsample_50_1595	Refined_42	6	S1
3_GI_subsample_50_2032	Refined_31	5	S1
3_GI_subsample_100_797	Refined_89	5	S1
3_GI_subsample_100_4195	Refined_42	5	S1
3_GI_subsample_75_917	Refined_52	5	S1
3_GI_subsample_100_4380	Refined_42	5	S1
3_GI_subsample_75_2742	Refined_4	5	S1
3_GI_subsample_75_151	Refined_35	5	S1
3_GI_subsample_50_670	Refined_59	5	S1
3_GI_subsample_75_216	Refined_2	5	S1
3_GI_subsample_100_197	Refined_2	5	S1

'''